@@ -8,7 +8,7 @@
 #########################################################################
 
 package Term::Clui;
-$VERSION = '1.67';   # 
+$VERSION = '1.70';   # ask_filename() uses readline to display its own prompt
 my $stupid_bloody_warning = $VERSION;  # circumvent -w warning
 require Exporter;
 @ISA = qw(Exporter);
@@ -42,7 +42,7 @@ if ($ENV{'CLUI_SPEAK'}) {  # 1.62 emacspeak not very relevant as a criterion
 	}
 	$Eflite = &which('eflite');
 	$Espeak = &which('espeak');
-	if ($Eflite) {
+	if ($Eflite && !$Espeak) {   # 1.68 Espeak should be the default
 		if (open($Eflite_FH,'|-',$Eflite)) {
 			select((select($Eflite_FH), $| = 1)[$[]); print $Eflite_FH q{};
 		} else {
@@ -142,6 +142,16 @@ sub getch {
 		if ($c eq '5') { getc_wrapper(0); return($KEY_PPAGE); }
 		if ($c eq '6') { getc_wrapper(0); return($KEY_NPAGE); }
 		if ($c eq 'Z') { return($KEY_BTAB); }
+		if ($c eq 'O') {   # 1.68 Haiku wierdness, inherited from an old Suse
+			$c = getc_wrapper(0);
+			if ($c eq 'A') { return($KEY_UP); }    # 1.68
+			if ($c eq 'B') { return($KEY_DOWN); }  # 1.68
+			if ($c eq 'C') { return($KEY_RIGHT); } # 1.68
+			if ($c eq 'D') { return($KEY_LEFT); }  # 1.68
+			if ($c eq 'F') { return($KEY_END); }   # 1.68
+			if ($c eq 'H') { return($KEY_HOME); }  # 1.68
+			return($c);
+		}
 		if ($c eq '[') {
 			$c = getc_wrapper(0);
 			if ($c eq 'A') { return($KEY_UP); }
@@ -381,7 +391,7 @@ sub endwin {
 # ----------------------- size handling ----------------------
 
 my ($maxcols, $maxrows); my $size_changed = 1;
-my ($otherlines, @otherlines, $notherlines);
+my @OtherLines;  # 20131002 $otherlines, $notherlines no longer global
 
 sub check_size {
 	if (! $size_changed) { return; }
@@ -395,9 +405,8 @@ sub check_size {
 	}
 	$maxcols = $maxcols || 80; $maxcols--;
 	$maxrows = $maxrows || 24;
-	if ($notherlines) {
-		@otherlines = &fmt($otherlines);
-		$notherlines = scalar @otherlines;
+	if (@OtherLines) {
+		@OtherLines = &fmt(join("\n",@OtherLines));
 	}
 	$size_changed = 0;
 }
@@ -416,11 +425,9 @@ sub ask_filename { my ($question, $default) = @_;  # 1.65 tab-completion
 		return ask($question, $default);
 	}
 	initscr(speakup_silent=>1);
-	my $nol = display_question($question);
 	endwin();
 	$term = new Term::ReadLine 'ProgramName';
-	# print STDERR "$question ";
-	my $filename = $term->readline('');
+	my $filename = $term->readline($question.' ');   # 1.70
 	print STDERR "\e[J";
 	$filename =~ s/ $//;   # 1.66
 	return $filename;
@@ -434,7 +441,7 @@ sub ask { my ($question, $default) = @_;
 	my $nol = &display_question($question);
 
 	my $i = 0; my $n = 0; my @s = (); # cursor position, length, string
-	if ($default) {
+	if (defined $default) {  # 1.69 defined, to include 0
 		&speak("$question, default is $default");
 		$default =~ s/\t/	/g;
 		@s = split(q{}, $default); $n = scalar @s; $i = $[;
@@ -521,9 +528,8 @@ sub choose {  my $question = shift; local @list = @_;  # @list must be local
 
 	&initscr(mouse_mode=>1, speakup_silent=>1);
 	&size_and_layout(0);
-	@otherlines = &fmt($otherlines);
-	$notherlines = scalar @otherlines;
-	my $speaktext = join(' ',$list[$this_cell],'. ',@otherlines);
+	@OtherLines = &fmt($otherlines);
+	my $speaktext = join(' ',$list[$this_cell],'. ',@OtherLines);
 	if (wantarray) {
 		$#marked = $#list;
 		if ($firstlinelength < $maxcols-30) {
@@ -550,7 +556,7 @@ sub choose {  my $question = shift; local @list = @_;  # @list must be local
 			&up(1); &clrtoeol(); &endwin(); $clue_has_been_given = 0;
 			return wantarray ? () : undef;
 		}
-		my $speaktext = join(' ',$list[$this_cell],'. ',@otherlines);
+		my $speaktext = join(' ',$list[$this_cell],'. ',@OtherLines);
 		&speak("choose, $speaktext");
 	}
 	&wr_screen();
@@ -604,11 +610,12 @@ sub choose {  my $question = shift; local @list = @_;  # @list must be local
 			$this_cell--; &wr_cell($this_cell+1); &wr_cell($this_cell); 
 			&speak($list[$this_cell]);
 		} elsif ((($c eq "j") || ($c == $KEY_DOWN)) && ($irow < $nrows)) {
-			$mid_col = $icol[$this_cell] + 0.5 * length($list[$this_cell]);
-			$left_of_target = 1000;
+			my $mid_col = $icol[$this_cell] + 0.5 * length($list[$this_cell]);
+			my $left_of_target = 1000;
 			for ($inew=$this_cell+1; $inew < $#list; $inew++) {
 				last if $icol[$inew] < $mid_col;	# skip rest of row
 			}
+			my $new_mid_col = 0;
 			for (; $inew < $#list; $inew++) {
 				$new_mid_col = $icol[$inew] + 0.5*length($list[$inew]);
 				last if $new_mid_col >= $mid_col;		# we've reached it
@@ -620,11 +627,12 @@ sub choose {  my $question = shift; local @list = @_;  # @list must be local
 			&wr_cell($iold); &wr_cell($this_cell);
 			&speak($list[$this_cell]);
 		} elsif ((($c eq "k") || ($c == $KEY_UP)) && ($irow > 1)) {
-			$mid_col = $icol[$this_cell] + 0.5*length($list[$this_cell]);
-			$right_of_target = 1000;
+			my $mid_col = $icol[$this_cell] + 0.5*length($list[$this_cell]);
+			my $right_of_target = 1000;
 			for ($inew=$this_cell-1; $inew > 0; $inew--) {
 				last if $irow[$inew] < $irow[$this_cell];	# skip rest of row
 			}
+			my $new_mid_col = 0;
 			for (; $inew > 0; $inew--) {
 				last unless $icol[$inew];
 				$new_mid_col = $icol[$inew] + 0.5*length($list[$inew]);
@@ -721,12 +729,12 @@ sub layout { my @list = @_;
 	return $irow;
 }
 sub wr_screen {
-	my $i;
-	for ($i=$[; $i<=$#list; $i++) {
+	for (my $i=$[; $i<=$#list; $i++) {
 		&wr_cell($i) unless $i==$this_cell;
 	}
+	my $notherlines = scalar @OtherLines;
 	if ($notherlines && ($nrows+$notherlines) < $maxrows) {
-		&puts("\r\n", join("\r\n", @otherlines), "\r");
+		&puts("\r\n", join("\r\n", @OtherLines), "\r");
 	}
 	&wr_cell($this_cell);
 }
@@ -973,9 +981,10 @@ sub edit {	my ($title, $text) = @_;
 		system $ENV{EDITOR} || "vi"; # should also look in ~/db/choices.db
 	} elsif ($argc == 2) {
 		# must create tmp file with title embedded in name
-		$tmpdir = '/tmp';
-		($safename = $title) =~ s/[\W_]+/_/g;
-		$file = "$tmpdir/$safename.$$";
+		my $tmpdir = '/tmp';
+		my $safename = $title;
+		$safename =~ s/[\W_]+/_/g;
+		my $file = "$tmpdir/$safename.$$";
 		if (!open(F,">$file")) {&sorry("can't open $file: $!\n");return q{};}
 		print F $text; close F;
 		$editor = $ENV{EDITOR} || "vi"; # should also look in ~/db/choices.db
@@ -1181,7 +1190,7 @@ sub speak {  my ($text, $wait) = @_;
 			# useless emacspeak op: tts_sy nc_state all 0 0  1 225\nq {[:np  ]}
 			if ($wait) { select(undef,undef,undef,0.3+0.07*length($text)); }
 		}
-	} elsif ($Espeak) {
+	} elsif ($Espeak) {  # 1.68 should be using Speech::eSpeak !
 		if ($Espeak_PID) { kill SIGHUP, $Espeak_PID; wait; $Espeak_PID = 0; }
 		$Espeak_PID = fork();
 		if ($Espeak_PID) {
@@ -1215,19 +1224,19 @@ sub speak {  my ($text, $wait) = @_;
 sub display_question {   my $question = shift; my %options = @_;
 	# used by &ask and &confirm, but not by &choose ...
 	&check_size();
-	my ($firstline, @otherlines);
+	my ($firstline, $otherlines);  # 20131002 @otherlines => $otherlines
 	if ($options{nofirstline}) {
-		@otherlines = &fmt($question);
+		@OtherLines = &fmt($question);
 	} else {
 		($firstline,$otherlines) = split(/\r?\n/, $question, 2);
-		@otherlines = &fmt($otherlines);
+		@OtherLines = &fmt($otherlines);
 		if ($firstline) { &puts("$firstline "); }
 	}
-	if (@otherlines) {
-		&puts("\r\n", join("\r\n", @otherlines), "\r");
+	if (@OtherLines) {
+		&puts("\r\n", join("\r\n", @OtherLines), "\r");
 		&goto(1 + length $firstline, 0);
 	}
-	return scalar @otherlines;
+	return scalar @OtherLines;
 }
 sub erase_lines {  # leaves cursor at beginning of line $_[$[]
 	&goto(0, $_[$[]); print TTY "\e[J";
@@ -1360,9 +1369,9 @@ The application needs no modification.
 
 There is an equivalent Python3 module,
 with (as far as possible) the same calling interface, at
-http://cpansearch.perl.org/src/PJB/Term-Clui-1.62/py/TermClui.py
+http://cpansearch.perl.org/src/PJB/Term-Clui-1.70/py/TermClui.py
 
-This is Term::Clui.pm version 1.67
+This is Term::Clui.pm version 1.70
 
 =head1 WINDOW-SIZE
 
@@ -1662,6 +1671,6 @@ which were in turn based on some even older curses-based programs in I<C>.
 
 There is an equivalent Python3 module,
 with (as far as possible) the same calling interface, at
-http://cpansearch.perl.org/src/PJB/Term-Clui-1.67/py/TermClui.py
+http://cpansearch.perl.org/src/PJB/Term-Clui-1.70/py/TermClui.py
 
 =cut
